--[[****************************************************************
	ItemAge v0.14

	Author: Evil Duck
	****************************************************************

	Plug-in for World of Warcraft

	****************************************************************]]


-- 0.14 WoW4
-- 0.12a Fixed broken key-binding
-- 0.12 Fixed a bug that would cause the auction-count in certain cases,
--      to not be updated, fixed bug with hidden age-display on items
--      that caused age to not be updated
-- 0.10 Added aution-count
-- 0.08 Fixed mailbox bug
-- 0.06 Added notes, fixed a bug in database purging
-- 0.04 Work-around for Blizzard premature VARIABLES_LOADED event, Bugfix in api
-- 0.03 Added function for retrieving age for use by other addons
-- 0.02 Release with age in tooltip
-- 0.01 Beta version


BINDING_HEADER_ITEMAGE = "ItemAge bindings";
BINDING_NAME_TOGGLEITEMAGE = "Toggle ItemAge display";
BINDING_NAME_ADDITEMCOMMENT = "Set item comment";
ITEMAGE_VERSIONTEXT = "ItemAge v0.14";
SLASH_ITEMAGE1 = "/itemage";
SLASH_ITEMAGE2 = "/age";
SLASH_ITEMAGE3 = "/ia";


ItemAge_Items={};
ItemAge_Info={};
local ItemAge_Tooltip=nil;
local ItemAge_TooltipText1=nil;
local ItemAge_Hook={ TT={} };
local ItemAge_CmtItemID=nil;
local ItemAge_CmtOldItem=nil;
local ItemAge_CmtNewItem=nil;
local ItemAge_CmtEditing=nil;
local ItemAge_AuctionList=nil;



-- Set up for handling
function ItemAge_OnLoad(self)
	self:RegisterEvent("BANKFRAME_OPENED");
	self:RegisterEvent("PLAYERBANKSLOTS_CHANGED");
	self:RegisterEvent("BAG_UPDATE");
	self:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED");
	self:RegisterEvent("GUILDBANKFRAME_OPENED");
	self:RegisterEvent("ADDON_LOADED");
	self:RegisterEvent("AUCTION_OWNED_LIST_UPDATE");
	self:RegisterEvent("AUCTION_HOUSE_SHOW");
	self:RegisterEvent("AUCTION_HOUSE_CLOSED");

	StaticPopupDialogs["ITEMAGE_SETCOMMENT"] = {
		text="Comment for this item:",
		OnShow = function()
			ItemAge_CmtEditing=true;
			if (ItemAge_CmtOldItem) then
				_G[self:GetName().."EditBox"]:SetText(ItemAge_CmtOldItem);
			else
				_G[self:GetName().."EditBox"]:SetText("");
			end
		end,
		OnAccept = function()
			ItemAge_CmtNewItem=_G[self:GetParent():GetName().."EditBox"]:GetText();
			if (ItemAge_CmtItemID and ItemAge_CmtItemID~=0) then
				if (ItemAge_CmtNewItem=="") then ItemAge_CmtNewItem=nil; end
				ItemAge_Items[ItemAge_CmtItemID].Comment=ItemAge_CmtNewItem;
			end
			ItemAge_CmtEditing=nil;
		end,
		OnCancel = function()
			ItemAge_CmtItemID=nil;
			ItemAge_CmtEditing=nil;
		end,
		hasEditBox = 1,
		button1=ACCEPT,
		button2=CANCEL,
		timeout=0,
		whileDead=1,
		hideOnEscape=1
	};

	SlashCmdList["ITEMAGE"]=function(msg) ItemAge_Slasher(msg) end;
end


function ItemAge_SetItemComment()
	StaticPopup_Show ("ITEMAGE_SETCOMMENT");
end


-- An event has been received
function ItemAge_OnEvent(self,event,a1)
	if (event=="BANKFRAME_OPENED") then ItemAge_BankOpen=true;
	elseif (event=="BANKFRAME_CLOSED") then ItemAge_BankOpen=nil; return; end

	if (event=="BAG_UPDATE" or event=="PLAYERBANKSLOTS_CHANGED" or event=="BANKFRAME_OPENED") then
		ItemAge_ReadStuff();
		return;
	end

	if (event=="GUILDBANKBAGSLOTS_CHANGED" or event=="GUILDBANKFRAME_OPENED") then
		ItemAge_ReadGuildBank();
		return;
	end

	if (event=="AUCTION_OWNED_LIST_UPDATE" or event=="AUCTION_HOUSE_SHOW") then
		ItemAge_ReadAuctions();
		return;
	end

	if (event=="AUCTION_HOUSE_CLOSED") then
		if (ItemAge_AuctionList) then
			-- Add count for new auctions
			for uID,uValue in pairs(ItemAge_AuctionList) do
				if (ItemAge_Items[uID]) then
					if (not ItemAge_Items[uID].Auction) then
						ItemAge_Items[uID].Auction={ Count=0 };
					end
					if (not ItemAge_Items[uID].Auction.Auction) then
						ItemAge_Items[uID].Auction.Count=ItemAge_Items[uID].Auction.Count+uValue;
					end
				end
			end
			-- Clear all "on auction" flags
			for uID,uTable in pairs(ItemAge_Items) do
				if (uTable.Auction) then uTable.Auction.Auction=nil; end
			end
			-- Set current "on auction" flags
			for uID,uValue in pairs(ItemAge_AuctionList) do
				if (ItemAge_Items[uID]) then
					ItemAge_Items[uID].Auction.Auction=true;
				end
			end
			ItemAge_AuctionList=nil;
		end
		return;
	end

	if (event=="ADDON_LOADED" and a1=="ItemAge") then
		ItemAge_Hook.TT.SetBagItem=GameTooltip.SetBagItem; GameTooltip.SetBagItem=ItemAge_Hook.SetBagItem;
		ItemAge_Hook.TT.SetGuildBankItem=GameTooltip.SetGuildBankItem; GameTooltip.SetGuildBankItem=ItemAge_Hook.SetGuildBankItem;
		ItemAge_Hook.TT.SetHyperlink=GameTooltip.SetHyperlink; GameTooltip.SetHyperlink=ItemAge_Hook.SetHyperlink;
		ItemAge_Hook.TT.SetInboxItem=GameTooltip.SetInboxItem; GameTooltip.SetInboxItem=ItemAge_Hook.SetInboxItem;
		ItemAge_Hook.TT.SetInventoryItem=GameTooltip.SetInventoryItem; GameTooltip.SetInventoryItem=ItemAge_Hook.SetInventoryItem;
		if (not ItemAge_Hook.TT.SetBagItem) then ItemAge_Chat("Hook is zero"); end
		if (not ItemAge_Info.Init) then ItemAge_Info={ Init=true, Bags=true, Bank=true, GBank=true }; end
		local now=time();
		local count=0;
		if (ItemAge_Items) then
			for uid,val in pairs(ItemAge_Items) do
				if (val.Bag) then
					if ((now-ItemAge_Items[uid].Last)>(100*24*60*60)) then ItemAge_Items[uid]=nil; count=count+1; end
				end
			end
		end
		if (count>0) then ItemAge_Chat("Purged "..count.." old items from ItemAge",0,1,0); end
		return;
	end
end


function ItemAge_Hook:SetInventoryItem(unit,slot,nameOnly)
	local hasItem,hasCooldown,repairCost=ItemAge_Hook.TT.SetInventoryItem(self,unit,slot,nameOnly);
	if (not ItemAge_Info.NoTooltip) then ItemAge_SetTooltipAge(); end
	return hasItem,hasCooldown,repairCost;
end
function ItemAge_Hook:SetInboxItem(index,aIndex)
	ItemAge_Hook.TT.SetInboxItem(self,index,aIndex);
	if (not ItemAge_Info.NoTooltip) then ItemAge_SetTooltipAge(); end
end
function ItemAge_Hook:SetHyperlink(link)
	ItemAge_Hook.TT.SetHyperlink(self,link);
	if (not ItemAge_Info.NoTooltip) then ItemAge_SetTooltipAge(); end
end
function ItemAge_Hook:SetGuildBankItem(tab,id)
	ItemAge_Hook.TT.SetGuildBankItem(self,tab,id);
	if (not ItemAge_Info.NoTooltip) then ItemAge_SetTooltipAge(); end
end
function ItemAge_Hook:SetBagItem(bag,slot)
	local hasCooldown,repairCost=ItemAge_Hook.TT.SetBagItem(self,bag,slot);
	if (not ItemAge_Info.NoTooltip) then ItemAge_SetTooltipAge(); end
	return hasCooldown,repairCost;
end

function ItemAge_ReadAuctions()
	if (not ItemAge_Items) then return; end

	ItemAge_AuctionList=nil;
	ItemAge_AuctionList={};

	-- Check all own auctions
	local index=1;
	while (GetAuctionItemLink("owner",index)) do
		local UniqueID=ItemAge_GetUniqueID(GetAuctionItemLink("owner",index));
		if (UniqueID~=0 and ItemAge_Items[UniqueID]) then
			if (not ItemAge_AuctionList[UniqueID]) then ItemAge_AuctionList[UniqueID]=0; end
			ItemAge_AuctionList[UniqueID]=ItemAge_AuctionList[UniqueID]+1;
		end
		index=index+1;
	end
end

function ItemAge_ReadGuildBank()
	local thistab=GetCurrentGuildBankTab();
	local now=time();

	local column=1;
	local slot;
	local index=1;

	local button,link;
	while(column<=7) do
		slot=1;
		while(slot<=14) do
			local bname="GuildBankColumn"..column.."Button"..slot;
			button=_G[bname];
			if (not button) then return; end
			if (not button.ItemAge) then button.ItemAge=CreateFrame("Frame",button:GetName().."_Info",button,"ItemAge_AgeTemplate"); end
			link=GetGuildBankItemLink(thistab,index);
			if (link) then ItemAge_HandleUID(nil,link,button,now,ItemAge_Info.GBank);
			elseif (button.ItemAge) then button.ItemAge:Hide(); end
			slot=slot+1;
			index=index+1;
		end
		column=column+1;
	end
end


function ItemAge_ReadStuff()
	local link,bagslots;
	local now=time();

	-- Do all bank slots
	for item=1,28 do
		link=GetContainerItemLink(BANK_CONTAINER,item);
		local button=_G["BankFrameItem"..item];
		if (not button.ItemAge) then button.ItemAge=CreateFrame("Frame",button:GetName().."_Info",button,"ItemAge_AgeTemplate"); end
		if (link) then ItemAge_HandleUID(nil,link,button,now,ItemAge_Info.Bank);
		elseif (button.ItemAge) then button.ItemAge:Hide(); end
	end

	-- Each banker bag
	for num=5,11 do
		bagslots=GetContainerNumSlots(num);
		-- Each slot in said bag
		link=nil;
		for item=1,bagslots do
			local button=_G["ContainerFrame".. num+1 .."Item".. bagslots-(item-1)];
			if (not button.ItemAge) then button.ItemAge=CreateFrame("Frame",button:GetName().."_Info",button,"ItemAge_AgeTemplate"); end
			link=GetContainerItemLink(num,item);
			if (link) then ItemAge_HandleUID(nil,link,button,now,ItemAge_Info.Bank);
			elseif (button.ItemAge) then button.ItemAge:Hide(); end
		end
	end

	-- Save bags
	for num=0,4 do
		local bagslots=GetContainerNumSlots(num);
		local checker;
		for item=1,bagslots do
			local button=_G["ContainerFrame".. num+1 .."Item".. bagslots-(item-1)];
			if (not button.ItemAge) then button.ItemAge=CreateFrame("Frame",button:GetName().."_Info",button,"ItemAge_AgeTemplate"); end
			link=GetContainerItemLink(num,item);
			if (link) then ItemAge_HandleUID(true,link,button,now,ItemAge_Info.Bags);
			elseif (button.ItemAge) then button.ItemAge:Hide(); end
		end
	end
end

function ListTableContents(level,chunk)
	local pre="";
	while (level>1) do pre=pre.."    "; level=level-1; end
	for thisname,tEntry in pairs(chunk) do
		ItemAge_Chat("Contents: "..thisname.." - "..type(tEntry),1);
		if (type(tEntry)=="table") then ListTableContents(level+1,tEntry); end
	end
end


function ItemAge_HandleUID(bags,link,button,now,showit)
	if (ItemAge_Info.Hidden) then showit=nil; end
	local UniqueID=ItemAge_GetUniqueID(link);
	if (UniqueID~=0 and not ItemAge_Items[UniqueID]) then
		ItemAge_Items[UniqueID]={ First=now, Last=now };
		_G[button.ItemAge:GetName().."_AgeTL"]:SetText("New");
		if (showit) then button.ItemAge:Show(); else button.ItemAge:Hide(); end
	elseif (UniqueID~=0) then
		ItemAge_Items[UniqueID].Last=now;
		_G[button.ItemAge:GetName().."_AgeTL"]:SetText(ItemAge_GetAgeText(now-ItemAge_Items[UniqueID].First,true));
		if (showit) then button.ItemAge:Show(); else button.ItemAge:Hide(); end
	else
		button.ItemAge:Hide();
	end
	if (ItemAge_Items[UniqueID]) then
		ItemAge_Items[UniqueID].Bag=bags;
		-- The following line assumes the item is not in the auction-house
		if (ItemAge_Items[UniqueID].Auction) then ItemAge_Items[UniqueID].Auction.Auction=nil; end
	end
end


function ItemAge_GetAge(link)
	local UID=ItemAge_GetUniqueID(link);
	if (not UID) then return nil; end
	if (not ItemAge_Items[UID]) then return nil; end
	return time()-ItemAge_Items[UID].First;
end


function ItemAge_GetAgeText(age,short)
	local text=string.format("%.0f",age); if (short) then text=text.."s"; else text=text.." seconds"; end
	if (age>=60) then
		age=age/60;
		text=string.format("%.0f",age); if (short) then text=text.."m"; else text=text.." minutes"; end
		if (age>=60) then
			age=age/60;
			text=string.format("%.0f",age); if (short) then text=text.."h"; else text=text.." hours"; end
			if (age>=24) then
				age=age/24;
				local formatstring="%.1f";
				if (age>7) then formatstring="%.0f"; end
				text=string.format(formatstring,age); if (short) then text=text.."d"; else text=text.." days"; end
			end
		end
	end
	return text;
end


function ItemAge_GetUniqueID(link)
	if (type(link)~="string") then
		if (type(link)~="number") then return 0; end
		_,link=GetItemInfo(link);
	end
	local itemID,ench,g1,g2,g3,g4,suff,uid=link:match("|Hitem:(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+)|h%[(.-)%]|h");
	if (not uid) then
		itemID,ench,g1,g2,g3,g4,suff,uid=link:match("item:(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+)");
	end
	if (not uid) then return 0; end
	return tonumber(uid),uid;
end


function ItemAge_OnUpdate(self,elapsed)
	if (ItemAge_Info.NoTooltip) then return; end
--	if (GameTooltip) then
--		if (GameTooltip:IsVisible()) then
--			local _,ThisItem=GameTooltip:GetItem();
--			if (ThisItem==ItemAge_Tooltip) then return; end
--			if (ItemAge_Tooltip) then
--				ItemAge_SetTooltipAge();
--			end
--		else
--			ItemAge_Tooltip=nil;
--			ItemAge_TooltipText1=nil;
--		end
--	end
end


function ItemAge_SetTooltipAge()
	local _,ThisItem=GameTooltip:GetItem();
	ItemAge_Tooltip=ThisItem;
	if (ItemAge_Tooltip) then
		local UniqueID=ItemAge_GetUniqueID(ItemAge_Tooltip);
		if (not UniqueID) then return; end
		if (UniqueID~=0 and ItemAge_Items[UniqueID]) then
			-- Insert age
			GameTooltip:AddDoubleLine("ItemAge: "..ItemAge_GetAgeText(time()-ItemAge_Items[UniqueID].First),"",.3,.3,1,1,1,1);
			if (not ItemAge_CmtEditing) then
				ItemAge_CmtItemID=UniqueID;
				ItemAge_CmtOldItem=ItemAge_Items[UniqueID].Comment;
			end
			-- Insert auction-info
			if (ItemAge_Items[UniqueID].Auction and ItemAge_Items[UniqueID].Auction.Count>0) then
				GameTooltip:AddLine("Auctioned "..ItemAge_Items[UniqueID].Auction.Count.." times",.6,.6,1,1);	-- 1=wrap text
			end
			-- Insert comment
			if (ItemAge_Items[UniqueID].Comment) then
				GameTooltip:AddLine(ItemAge_Items[UniqueID].Comment,.6,.6,1,1);	-- 1=wrap text
			end
		elseif (UniqueID==0) then
			GameTooltip:AddLine("ItemAge: Probably vendor-bought",.3,.3,1,1);	-- 1=wrap text
		end
		GameTooltip:Show();
	end
end

function ItemAge_ToggleHidden()
	if (ItemAge_Info.Hidden) then ItemAge_Info.Hidden=nil; else ItemAge_Info.Hidden=true; end
	ItemAge_ReadStuff();
end

-- Generic chat-pane stuff
function ItemAge_Chat(msg,r,g,b)
	if (DEFAULT_CHAT_FRAME) then
		if (not r and not g and not b) then r=1; g=1; b=1; end;
		if (not r) then r=0; end; if (not g) then g=0; end; if (not b) then b=0; end;
		DEFAULT_CHAT_FRAME:AddMessage("IA: "..msg,r,g,b);
	end
end


-- There's slashing to be done
function ItemAge_Slasher(msg)
	if (not msg) then msg=""; end
	if (strlen(msg)>0) then msg=strlower(msg); end

	if (msg=="bags") then if (ItemAge_Info.Bags) then ItemAge_Info.Bags=nil; else ItemAge_Info.Bags=true; end ItemAge_ReadStuff(); return; end
	if (msg=="bank") then if (ItemAge_Info.Bank) then ItemAge_Info.Bank=nil; else ItemAge_Info.Bank=true; end ItemAge_ReadStuff(); return; end
	if (msg=="gbank") then if (ItemAge_Info.GBank) then ItemAge_Info.GBank=nil; else ItemAge_Info.GBank=true; end ItemAge_ReadGuildBank(); return; end
	if (msg=="tooltip") then if (ItemAge_Info.NoTooltip) then ItemAge_Info.NoTooltip=nil; else ItemAge_Info.NoTooltip=true; end return; end

	ItemAge_Chat(ITEMAGE_VERSIONTEXT);
	ItemAge_Chat(" -> "..SLASH_ITEMAGE2.." tooltip => Toggle diplay of item age in tooltips.");
	ItemAge_Chat("Toggle display of item ages in your:");
	ItemAge_Chat(" -> "..SLASH_ITEMAGE2.." bags => bags");
	ItemAge_Chat(" -> "..SLASH_ITEMAGE2.." bank => bank");
	ItemAge_Chat(" -> "..SLASH_ITEMAGE2.." gbank => guild bank");
	ItemAge_Chat("PLEASE NOTE: The WoW guild bank does too often not report correct info for items, resulting in no age shown for these items.");
end
