--[[****************************************************************
	Cursor Info v0.01

	Author: Evil Duck
	****************************************************************

	Plug-in for World of Warcraft

	****************************************************************]]

-- 0.01 Beta version


Cursor_Info_Items={};
local Cursor_Info_ShowingItem=0;


CURSOR_INFO_VERSIONTEXT = "Cursor Information v0.01";


-- Set up for handling
function Cursor_Info_OnLoad()
	this:RegisterEvent("BANKFRAME_OPENED");
	this:RegisterEvent("BAG_UPDATE");
end


-- An event has been received
function Cursor_Info_OnEvent(event)
	if (event=="BANKFRAME_OPENED") then Cursor_Info_BankOpen=true;
	elseif (event=="BANKFRAME_CLOSED") then Cursor_Info_BankOpen=nil; return; end

	if (event=="BAG_UPDATE" or event=="BANKFRAME_OPENED") then
		Cursor_Info_ReadStuff();
		return; 
	end
end


function Cursor_Info_ReadStuff()
	local link,bagslots;
	local now=time();

	-- Do all bank slots
	for num=1,28 do
		link=GetContainerItemLink(BANK_CONTAINER,num);
		if (link) then
			local UniqueID=Cursor_Info_GetUniqueID(link)
			if (UniqueID~=0 and not Cursor_Info_Items[UniqueID]) then
				Cursor_Info_Items[UniqueID]=now;
				LootCount_Chat(UniqueID.." added");
			end
		end
	end

	-- Each banker bag
	for num=5,11 do
		bagslots=GetContainerNumSlots(num);
		-- Each slot in said bag
		link=nil;
		for item=1,bagslots do
			link=GetContainerItemLink(num,item);
			if (link) then
				local UniqueID=Cursor_Info_GetUniqueID(link)
				if (UniqueID~=0 and not Cursor_Info_Items[UniqueID]) then
					Cursor_Info_Items[UniqueID]=now;
					LootCount_Chat(UniqueID.." added");
				end
			end
		end
	end

	-- Save bags
	for num=0,4 do
		local bagslots=GetContainerNumSlots(num);
		link=nil;
		for item=1,bagslots do
			link=GetContainerItemLink(num,item);
			if (link) then
				local UniqueID=Cursor_Info_GetUniqueID(link)
				if (UniqueID~=0 and not Cursor_Info_Items[UniqueID]) then
					Cursor_Info_Items[UniqueID]=now;
					LootCount_Chat(UniqueID.." added");
				end
			end
		end
	end
end


function Cursor_Info_GetUniqueID(link)
	if (type(link)~="string") then
		if (type(link)~="number") then return nil,nil; end
		_,link=GetItemInfo(link);
	end
	local itemID,ench,g1,g2,g3,g4,suff,uid=link:match("|Hitem:(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+)|h%[(.-)%]|h");
	if (not uid) then
		itemID,ench,g1,g2,g3,g4,suff,uid=link:match("item:(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+):(%p?%d+)");
	end
	if (not uid) then return 0; end
	return tonumber(uid);
end


function Cursor_Info_SetPosition()
	local scale,x,y=UIParent:GetScale(),GetCursorPosition();
	Cursor_Info_Frame:ClearAllPoints();
	Cursor_Info_Frame:SetPoint("TOPLEFT",UIParent,"BOTTOMLEFT",(x+16 -(Cursor_Info_Frame:GetWidth()/2))/scale,(y-15)/scale);
--	if (LootCount_Chat) then LootCount_Chat("Setting CI: "..string.format("%.1f",x+16)..","..string.format("%.1f",y+32)); end
end


function Cursor_Info_OnUpdate(elapsed)
	if (not Cursor_Info_Frame:IsVisible()) then
		if (UnitExists("mouseover")) then
			if (LootCount_DropCount_DB) then
				if (LootCount_DropCount_DB.Count) then
					local unit=UnitName("mouseover");
					local skinned,killed;
					if (LootCount_DropCount_DB.Count[unit]) then
						if (LootCount_DropCount_DB.Count[unit].Skinning) then skinned="Skinned "..LootCount_DropCount_DB.Count[unit].Skinning.." times"; end
						if (LootCount_DropCount_DB.Count[unit].Kill) then killed=LootCount_DropCount_DB.Count[unit].Kill.." kills"; end
						if (killed or skinned) then
							Cursor_Info_Frame:SetOwner(UIParent,"ANCHOR_BOTTOMLEFT");
							local first=skinned;
							local second="";
							if (killed) then second=skinned; first=killed; end
							Cursor_Info_Frame:AddDoubleLine(first,"",.4,.4,1,1,0,1);
							Cursor_Info_Frame:AddDoubleLine(second,"",.4,.4,1,1,0,1);
							Cursor_Info_SetPosition();
							Cursor_Info_Frame:Show();
						end
					end
				end
			end
		elseif (GameTooltip and GameTooltip:IsVisible()) then
			local _,ThisItem=GameTooltip:GetItem();
			local UniqueID=Cursor_Info_GetUniqueID(ThisItem)
			if (UniqueID~=0 and Cursor_Info_Items[UniqueID] and Cursor_Info_ShowingItem~=UniqueID) then
				local age=time()-Cursor_Info_Items[UniqueID];
				local text=age.." seconds";
				Cursor_Info_Frame:SetOwner(UIParent,"ANCHOR_BOTTOMLEFT");
				if (age>=60) then
					age=age/60;
					text=age.." minutes";
					if (age>=60) then
						age=age/60;
						text=age.." hours";
						if (age>=24) then
							age=age/24;
							text=age.." days";
						end
					end
				end
				Cursor_Info_Frame:AddDoubleLine(text,"",.4,.4,1,1,0,1);
				Cursor_Info_SetPosition();
				Cursor_Info_ShowingItem=UniqueID;
				Cursor_Info_Frame:Show();
			end

			Cursor_Info_SetPosition();
			Cursor_Info_Frame:Show();
		end
	else
		if (not UnitExists("mouseover")) then
			Cursor_Info_Frame:Hide();
		elseif (not GameTooltip:IsVisible()) then 
			Cursor_Info_ShowingItem=0;
			Cursor_Info_Frame:Hide();
		end
	end
	if (Cursor_Info_Frame:IsVisible()) then Cursor_Info_SetPosition(); end
end
